@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2013-2020 The Octave Project Developers
+// Copyright (C) 2013-2021 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -582,28 +582,40 @@ namespace octave
     if (prevline.contains (case_key) && do_smart_indent)
       {
         QString last_line = text (line-1);
+        int prev_ind = indentation (line-1);
         int act_ind = indentation (line);
 
-        if (last_line.contains ("switch"))
+        if (last_line.contains (QRegExp ("^[\t ]*switch")))
           {
             indent (line+1);
             act_ind = indentation (line+1);
           }
         else
-          unindent (line);
+          {
+            if (prev_ind == act_ind)
+              unindent (line);
+            else if (prev_ind > act_ind)
+              act_ind = prev_ind;
+          }
 
         setIndentation (line+1, act_ind);
         setCursorPosition (line+1, act_ind);
       }
 
     ekey = QRegExp ("^[\t ]*(?:end|endif|endfor|endwhile|until|endfunction"
-                    "|end_try_catch|end_unwind_protect)[\r]?[\t #%\n(;]");
+                    "|endswitch|end_try_catch|end_unwind_protect)[\r]?[\t #%\n(;]");
     if (prevline.contains (ekey))
       {
         if (indentation (line-1) <= indentation (line))
           {
             unindent (line+1);
             unindent (line);
+            if (prevline.contains ("endswitch"))
+              {
+                // endswitch has to me unndented twice
+                unindent (line+1);
+                unindent (line);
+              }
             setCursorPosition (line+1,
                                indentation (line));
           }
@@ -633,7 +645,6 @@ namespace octave
 
     QRegExp mid_block_regexp
       = QRegExp ("^[\t ]*(?:elseif|else"
-                 "|otherwise"
                  "|unwind_protect_cleanup|catch)"
                  "[\r\n\t #%]");
 
@@ -798,8 +809,12 @@ namespace octave
 
     // Split contents into single lines and complete commands
     QStringList lines = selectedText ().split (QRegExp ("[\r\n]"),
+#if defined (HAVE_QT_SPLITBEHAVIOR_ENUM)
+                                               Qt::SkipEmptyParts);
+#else
                                                QString::SkipEmptyParts);
-    for (int i = 0; i < lines.count (); i++)
+#endif
+  for (int i = 0; i < lines.count (); i++)
       {
         QString line = lines.at (i);
         if (line.trimmed ().isEmpty ())
@@ -1199,7 +1214,7 @@ namespace octave
 
     bool autofill_simple_end = (auto_endif == 2);
 
-    size_t start = line.toStdString ().find_first_not_of (" \t");
+    std::size_t start = line.toStdString ().find_first_not_of (" \t");
 
     // Check if following line has the same or less indentation
     // Check if the following line does not start with
@@ -1208,7 +1223,7 @@ namespace octave
       {
         int offset = 2;     // linenr is the old line, thus, linnr+1 is the
                             // new one and can not be taken into account
-        size_t next_start;
+        std::size_t next_start;
         QString next_line;
 
         do                            // find next non-blank line
